@@ -13,7 +13,7 @@ from .lib.happierfuntokenizing import Tokenizer #Potts tokenizer
 try:
     from langid.langid import LanguageIdentifier, model
 except ImportError:
-    dlac.warn("Cannot import langid (cannot use addLanguageFilterTable)")
+    dlac.warn("Warning: Cannot import langid (cannot use addLanguageFilterTable)")
     pass
 
 class MessageAnnotator(DLAWorker):
@@ -39,12 +39,12 @@ class MessageAnnotator(DLAWorker):
         new_table = self.corptable + "_dedup"
         drop = """DROP TABLE IF EXISTS %s""" % (new_table)
         create = """CREATE TABLE %s like %s""" % (new_table, self.corptable)
-        mm.execute(self.corpdb, self.dbCursor, drop, charset=self.encoding, use_unicode=self.use_unicode)
-        mm.execute(self.corpdb, self.dbCursor, create, charset=self.encoding, use_unicode=self.use_unicode)
-        mm.standardizeTable(self.corpdb, self.dbCursor, new_table, collate=dlac.DEF_COLLATIONS[self.encoding.lower()], engine=dlac.DEF_MYSQL_ENGINE, charset=self.encoding, use_unicode=self.use_unicode)
+        mm.execute(self.corpdb, self.dbCursor, drop, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
+        mm.execute(self.corpdb, self.dbCursor, create, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
+        mm.standardizeTable(self.corpdb, self.dbCursor, new_table, collate=dlac.DEF_COLLATIONS[self.encoding.lower()], engine=dlac.DEF_MYSQL_ENGINE, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
 
         #Find column names:
-        columnNames = list(mm.getTableColumnNameTypes(self.corpdb, self.dbCursor, self.corptable, charset=self.encoding, use_unicode=self.use_unicode).keys())
+        columnNames = list(mm.getTableColumnNameTypes(self.corpdb, self.dbCursor, self.corptable, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file).keys())
         messageIndex = columnNames.index(self.message_field)
         try:
             retweetedStatusIdx = columnNames.index("retweeted_status_text")
@@ -54,7 +54,7 @@ class MessageAnnotator(DLAWorker):
 
         #find all groups that are not already inserted
         usql = """SELECT %s FROM %s GROUP BY %s""" % (self.correl_field, self.corptable, self.correl_field)
-        cfRows = [r[0] for r in mm.executeGetList(self.corpdb, self.dbCursor, usql, charset=self.encoding, use_unicode=self.use_unicode)]
+        cfRows = [r[0] for r in mm.executeGetList(self.corpdb, self.dbCursor, usql, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)]
         dlac.warn("deduplicating messages for %d '%s's"%(len(cfRows), self.correl_field))
 
         # if message level analysis
@@ -70,7 +70,7 @@ class MessageAnnotator(DLAWorker):
 
             # get msgs for groups:
             sql = """SELECT %s from %s where %s IN ('%s')""" % (','.join(columnNames), self.corptable, self.correl_field, "','".join(str(g) for g in groups))
-            rows = list(mm.executeGetList(self.corpdb, self.dbCursor, sql, warnQuery=False, charset=self.encoding, use_unicode=self.use_unicode))
+            rows = list(mm.executeGetList(self.corpdb, self.dbCursor, sql, warnQuery=False, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file))
             rows = [row for row in rows if row[messageIndex] and not row[messageIndex].isspace()]
 
             bf = []
@@ -108,7 +108,7 @@ class MessageAnnotator(DLAWorker):
                     continue
             if len(rows_to_write) >= dlac.MYSQL_BATCH_INSERT_SIZE:
                 sql = """INSERT INTO """+new_table+""" ("""+', '.join(columnNames)+""") VALUES ("""  +", ".join(['%s']*len(columnNames)) + """)"""
-                mm.executeWriteMany(self.corpdb, self.dbCursor, sql, rows_to_write, writeCursor=self.dbConn.cursor(), charset=self.encoding, use_unicode=self.use_unicode)
+                mm.executeWriteMany(self.corpdb, self.dbCursor, sql, rows_to_write, writeCursor=self.dbConn.cursor(), charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
                 rows_to_write = []
 
             if (counter % 500 == 0):
@@ -117,7 +117,7 @@ class MessageAnnotator(DLAWorker):
 
         if rows_to_write:
             sql = """INSERT INTO """+new_table+""" ("""+', '.join(columnNames)+""") VALUES ("""  +", ".join(['%s']*len(columnNames)) + """)"""
-            mm.executeWriteMany(self.corpdb, self.dbCursor, sql, rows_to_write, writeCursor=self.dbConn.cursor(), charset=self.encoding, use_unicode=self.use_unicode)
+            mm.executeWriteMany(self.corpdb, self.dbCursor, sql, rows_to_write, writeCursor=self.dbConn.cursor(), charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
 
     def addAnonymizedTable(self):
         """
@@ -126,12 +126,12 @@ class MessageAnnotator(DLAWorker):
         new_table = self.corptable + "_an"
         drop = """DROP TABLE IF EXISTS %s""" % (new_table)
         create = """CREATE TABLE %s like %s""" % (new_table, self.corptable)
-        mm.execute(self.corpdb, self.dbCursor, drop, charset=self.encoding, use_unicode=self.use_unicode)
-        mm.execute(self.corpdb, self.dbCursor, create, charset=self.encoding, use_unicode=self.use_unicode)
-        mm.standardizeTable(self.corpdb, self.dbCursor, new_table, collate=dlac.DEF_COLLATIONS[self.encoding.lower()], engine=dlac.DEF_MYSQL_ENGINE, charset=self.encoding, use_unicode=self.use_unicode)
+        mm.execute(self.corpdb, self.dbCursor, drop, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
+        mm.execute(self.corpdb, self.dbCursor, create, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
+        mm.standardizeTable(self.corpdb, self.dbCursor, new_table, collate=dlac.DEF_COLLATIONS[self.encoding.lower()], engine=dlac.DEF_MYSQL_ENGINE, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
 
         #Find column names:
-        columnNames = list(mm.getTableColumnNameTypes(self.corpdb, self.dbCursor, self.corptable, charset=self.encoding, use_unicode=self.use_unicode).keys())
+        columnNames = list(mm.getTableColumnNameTypes(self.corpdb, self.dbCursor, self.corptable, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file).keys())
         messageIndex = columnNames.index(self.message_field)
         try:
             retweetedStatusIdx = columnNames.index("retweeted_status_text")
@@ -141,7 +141,7 @@ class MessageAnnotator(DLAWorker):
 
         #find all groups that are not already inserted
         usql = """SELECT %s FROM %s GROUP BY %s""" % (self.correl_field, self.corptable, self.correl_field)
-        cfRows = [r[0] for r in mm.executeGetList(self.corpdb, self.dbCursor, usql, charset=self.encoding, use_unicode=self.use_unicode)]
+        cfRows = [r[0] for r in mm.executeGetList(self.corpdb, self.dbCursor, usql, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)]
         dlac.warn("anonymizing messages for %d '%s's"%(len(cfRows), self.correl_field))
 
         groupsAtTime = 1
@@ -151,7 +151,7 @@ class MessageAnnotator(DLAWorker):
 
             # get msgs for groups:
             sql = """SELECT %s from %s where %s IN ('%s')""" % (','.join(columnNames), self.corptable, self.correl_field, "','".join(str(g) for g in groups))
-            rows = list(mm.executeGetList(self.corpdb, self.dbCursor, sql, warnQuery=False, charset=self.encoding, use_unicode=self.use_unicode))
+            rows = list(mm.executeGetList(self.corpdb, self.dbCursor, sql, warnQuery=False, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file))
             rows = [row for row in rows if row[messageIndex] and not row[messageIndex].isspace()]
 
             for row in rows:
@@ -166,7 +166,7 @@ class MessageAnnotator(DLAWorker):
                     continue
             if len(rows_to_write) >= dlac.MYSQL_BATCH_INSERT_SIZE:
                 sql = """INSERT INTO """+new_table+""" ("""+', '.join(columnNames)+""") VALUES ("""  +", ".join(['%s']*len(columnNames)) + """)"""
-                mm.executeWriteMany(self.corpdb, self.dbCursor, sql, rows_to_write, writeCursor=self.dbConn.cursor(), charset=self.encoding, use_unicode=self.use_unicode)
+                mm.executeWriteMany(self.corpdb, self.dbCursor, sql, rows_to_write, writeCursor=self.dbConn.cursor(), charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
                 rows_to_write = []
 
             if (counter % 500 == 0):
@@ -175,7 +175,7 @@ class MessageAnnotator(DLAWorker):
 
         if rows_to_write:
             sql = """INSERT INTO """+new_table+""" ("""+', '.join(columnNames)+""") VALUES ("""  +", ".join(['%s']*len(columnNames)) + """)"""
-            mm.executeWriteMany(self.corpdb, self.dbCursor, sql, rows_to_write, writeCursor=self.dbConn.cursor(), charset=self.encoding, use_unicode=self.use_unicode)
+            mm.executeWriteMany(self.corpdb, self.dbCursor, sql, rows_to_write, writeCursor=self.dbConn.cursor(), charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
 
 
     def addSpamFilterTable(self, threshold=dlac.DEF_SPAM_FILTER):
@@ -195,19 +195,19 @@ class MessageAnnotator(DLAWorker):
         drop = """DROP TABLE IF EXISTS %s""" % (new_table)
         create = """CREATE TABLE %s like %s""" % (new_table, self.corptable)
         add_colum = """ALTER TABLE %s ADD COLUMN is_spam INT(2) NULL""" % (new_table)
-        mm.execute(self.corpdb, self.dbCursor, drop, charset=self.encoding, use_unicode=self.use_unicode)
-        mm.execute(self.corpdb, self.dbCursor, create, charset=self.encoding, use_unicode=self.use_unicode)
-        mm.execute(self.corpdb, self.dbCursor, add_colum, charset=self.encoding, use_unicode=self.use_unicode)
-        mm.standardizeTable(self.corpdb, self.dbCursor, new_table, collate=dlac.DEF_COLLATIONS[self.encoding.lower()], engine=dlac.DEF_MYSQL_ENGINE, charset=self.encoding, use_unicode=self.use_unicode)
+        mm.execute(self.corpdb, self.dbCursor, drop, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
+        mm.execute(self.corpdb, self.dbCursor, create, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
+        mm.execute(self.corpdb, self.dbCursor, add_colum, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
+        mm.standardizeTable(self.corpdb, self.dbCursor, new_table, collate=dlac.DEF_COLLATIONS[self.encoding.lower()], engine=dlac.DEF_MYSQL_ENGINE, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
 
         #Find column names:
-        columnNames = list(mm.getTableColumnNameTypes(self.corpdb, self.dbCursor, self.corptable, charset=self.encoding, use_unicode=self.use_unicode).keys())
+        columnNames = list(mm.getTableColumnNameTypes(self.corpdb, self.dbCursor, self.corptable, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file).keys())
         insertColumnNames = columnNames + ['is_spam']
         messageIndex = columnNames.index(self.message_field)
 
         #find all groups that are not already inserted
         usql = """SELECT %s FROM %s GROUP BY %s""" % (self.correl_field, self.corptable, self.correl_field)
-        cfRows = [r[0] for r in mm.executeGetList(self.corpdb, self.dbCursor, usql, charset=self.encoding, use_unicode=self.use_unicode)]
+        cfRows = [r[0] for r in mm.executeGetList(self.corpdb, self.dbCursor, usql, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)]
         dlac.warn("Removing spam messages for %d '%s's"%(len(cfRows), self.correl_field))
 
         # if message level analysis
@@ -224,7 +224,7 @@ class MessageAnnotator(DLAWorker):
 
             # get msgs for groups:
             sql = """SELECT %s from %s where %s IN ('%s')""" % (','.join(columnNames), self.corptable, self.correl_field, "','".join(str(g) for g in groups))
-            rows = list(mm.executeGetList(self.corpdb, self.dbCursor, sql, warnQuery=False, charset=self.encoding, use_unicode=self.use_unicode))
+            rows = list(mm.executeGetList(self.corpdb, self.dbCursor, sql, warnQuery=False, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file))
             rows = [row for row in rows if row[messageIndex] and not row[messageIndex].isspace()]
 
             total_messages = float(len(rows))
@@ -248,7 +248,7 @@ class MessageAnnotator(DLAWorker):
 
             if len(rows_to_write) >= dlac.MYSQL_BATCH_INSERT_SIZE:
                 sql = """INSERT INTO """+new_table+""" ("""+', '.join(insertColumnNames)+""") VALUES ("""  +", ".join(['%s']*len(insertColumnNames)) + """)"""
-                mm.executeWriteMany(self.corpdb, self.dbCursor, sql, rows_to_write, writeCursor=self.dbConn.cursor(), charset=self.encoding, use_unicode=self.use_unicode)
+                mm.executeWriteMany(self.corpdb, self.dbCursor, sql, rows_to_write, writeCursor=self.dbConn.cursor(), charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
                 rows_to_write = []
 
             if (counter % 500 == 0):
@@ -257,7 +257,7 @@ class MessageAnnotator(DLAWorker):
 
         if rows_to_write:
             sql = """INSERT INTO """+new_table+""" ("""+', '.join(insertColumnNames)+""") VALUES ("""  +", ".join(['%s']*len(insertColumnNames)) + """)"""
-            mm.executeWriteMany(self.corpdb, self.dbCursor, sql, rows_to_write, writeCursor=self.dbConn.cursor(), charset=self.encoding, use_unicode=self.use_unicode)
+            mm.executeWriteMany(self.corpdb, self.dbCursor, sql, rows_to_write, writeCursor=self.dbConn.cursor(), charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
         print('%d users removed!' % (users_removed))
 
     # TODO: add nicer implementation
@@ -265,14 +265,14 @@ class MessageAnnotator(DLAWorker):
         if totalcount > 10*dlac.MAX_SQL_SELECT:
             for i in range(0,totalcount, dlac.MAX_SQL_SELECT):
                 sql = "SELECT * FROM %s limit %d, %d" % (messageTable, i, dlac.MAX_SQL_SELECT)
-                for m in mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode):
+                for m in mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file):
                     yield [i for i in m]
         else:
             sql = "SELECT * FROM %s" % messageTable
-            for m in mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode):
+            for m in mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file):
                 yield m
 
-    def addLanguageFilterTable(self, langs, cleanMessages, lowercase):
+    def addLanguageFilterTable(self, langs, cleanMessages, lowercase, lightEnglishFilter=False):
         """Filters all messages in corptable for a given language. Keeps messages if
         confidence is greater than 80%. Uses the langid library.
 
@@ -286,10 +286,13 @@ class MessageAnnotator(DLAWorker):
             convert message to all lowercase before running langid
         """
         identifier = LanguageIdentifier.from_modelstring(model, norm_probs=True)
+        if lightEnglishFilter: 
+            identifier.set_languages(['en', 'es'])
+            langs = ['en']
 
         new_table = self.corptable + "_%s"
 
-        columnNames = mm.getTableColumnNames(self.corpdb, self.corptable, charset=self.encoding, use_unicode=self.use_unicode)
+        columnNames = mm.getTableColumnNames(self.corpdb, self.corptable, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
         messageIndex = [i for i, col in enumerate(columnNames) if col.lower() == dlac.DEF_MESSAGE_FIELD.lower()][0]
         #messageIDindex = [i for i, col in enumerate(columnNames) if col.lower() == dlac.DEF_MESSAGEID_FIELD.lower()][0]
 
@@ -298,9 +301,9 @@ class MessageAnnotator(DLAWorker):
         for l, table in messageTables.items():
             drop = """DROP TABLE IF EXISTS %s""" % (table)
             create = """CREATE TABLE %s like %s""" % (table, self.corptable)
-            mm.execute(self.corpdb, self.dbCursor, drop, charset=self.encoding, use_unicode=self.use_unicode)
-            mm.execute(self.corpdb, self.dbCursor, create, charset=self.encoding, use_unicode=self.use_unicode)
-            mm.standardizeTable(self.corpdb, self.dbCursor, table, collate=dlac.DEF_COLLATIONS[self.encoding.lower()], engine=dlac.DEF_MYSQL_ENGINE, charset=self.encoding, use_unicode=self.use_unicode)
+            mm.execute(self.corpdb, self.dbCursor, drop, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
+            mm.execute(self.corpdb, self.dbCursor, create, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
+            mm.standardizeTable(self.corpdb, self.dbCursor, table, collate=dlac.DEF_COLLATIONS[self.encoding.lower()], engine=dlac.DEF_MYSQL_ENGINE, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
 
         #ITERATE THROUGH EACH MESSAGE WRITING THOSE THAT ARE ENGLISH
         messageDataToAdd = {l: list() for l in langs}
@@ -308,7 +311,7 @@ class MessageAnnotator(DLAWorker):
         totalMessages = 0
         totalMessagesKept = 0
         sql = """SELECT COUNT(*) FROM %s""" % self.corptable
-        totalMessagesInTable = mm.executeGetList(self.corpdb, self.dbCursor, sql)[0][0]
+        totalMessagesInTable = mm.executeGetList(self.corpdb, self.dbCursor, sql, mysql_config_file=self.mysql_config_file)[0][0]
 
         print("Reading %s messages" % ",".join([str(totalMessagesInTable)[::-1][i:i+3] for i in range(0,len(str(totalMessagesInTable)),3)])[::-1])
         memory_limit = dlac.MYSQL_BATCH_INSERT_SIZE if dlac.MYSQL_BATCH_INSERT_SIZE < totalMessagesInTable else totalMessagesInTable/20
@@ -320,6 +323,7 @@ class MessageAnnotator(DLAWorker):
             message = messageRow[messageIndex]
 
             try:
+                message = tc.removeNonAscii(message)
                 message = html.unescape(message)
                 messageRow[messageIndex] = message
 
@@ -343,11 +347,19 @@ class MessageAnnotator(DLAWorker):
             except TypeError as e:
                 print(("         Error, ignoring row %s" % str(messageRow)))
 
-            if lang in langs and conf > dlac.DEF_LANG_FILTER_CONF:
+            if lightEnglishFilter and ((lang == 'es' and conf < .95) or (lang == 'en' and conf > .60)):
+                lang = 'en'
                 messageDataToAdd[lang].append(messageRow)
                 messageDataCounts[lang] += 1
 
                 totalMessagesKept+=1
+
+            elif lang in langs and conf > dlac.DEF_LANG_FILTER_CONF:
+                messageDataToAdd[lang].append(messageRow)
+                messageDataCounts[lang] += 1
+
+                totalMessagesKept+=1
+
             else:
                 continue
 
@@ -355,7 +367,7 @@ class MessageAnnotator(DLAWorker):
                 #write messages every so often to clear memory
                 for l, messageData in messageDataToAdd.items():
                     sql = """INSERT INTO """+messageTables[l]+""" ("""+', '.join(columnNames)+""") VALUES ("""  +", ".join(['%s']*len(columnNames)) + """)"""
-                    mm.executeWriteMany(self.corpdb, self.dbCursor, sql, messageData, writeCursor=self.dbConn.cursor(), charset=self.encoding, use_unicode=self.use_unicode)
+                    mm.executeWriteMany(self.corpdb, self.dbCursor, sql, messageData, writeCursor=self.dbConn.cursor(), charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
                     messageDataToAdd[l] = list()
 
                 for l, nb in [(x[0], len(x[1])) for x in iter(messageDataToAdd.items())]:
@@ -368,7 +380,7 @@ class MessageAnnotator(DLAWorker):
             print("Adding final rows")
             for l, messageData in messageDataToAdd.items():
                 sql = """INSERT INTO """+messageTables[l]+""" ("""+', '.join(columnNames)+""") VALUES ("""  +", ".join(['%s']*len(columnNames)) + """)"""
-                mm.executeWriteMany(self.corpdb, self.dbCursor, sql, messageData, writeCursor=self.dbConn.cursor(), charset=self.encoding, use_unicode=self.use_unicode)
+                mm.executeWriteMany(self.corpdb, self.dbCursor, sql, messageData, writeCursor=self.dbConn.cursor(), charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
 
         print("Kept %d out of %d messages" % (totalMessagesKept, totalMessages))
         pprint({messageTables[l]: v for l, v in messageDataCounts.items()})
