@@ -8,6 +8,8 @@ import sys
 import os
 import re
 
+from io import StringIO
+
 #infrastructure
 from .dlaWorker import DLAWorker
 from . import dlaConstants as dlac
@@ -46,37 +48,41 @@ class MessageTransformer(DLAWorker):
 
     groupsAtTime = 100
 
-    def __createTable(self, tableName, modify=''):
+    def _createTable(self, tableName, modify='', modify_id=''):
         drop = """DROP TABLE IF EXISTS %s""" % (tableName)
-        mm.execute(self.corpdb, self.dbCursor, drop, charset=self.encoding, use_unicode=self.use_unicode)
+        mm.execute(self.corpdb, self.dbCursor, drop, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
 
         sql = """CREATE TABLE %s like %s""" % (tableName, self.corptable)
-        mm.execute(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode)
+        mm.execute(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
 
         if modify:
             alter = """ALTER TABLE %s MODIFY %s %s""" % (tableName, self.message_field, modify)
+            mm.execute(self.corpdb, self.dbCursor, alter, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
+
+        if modify_id:#modify message id field:
+            alter = """ALTER TABLE %s MODIFY %s %s""" % (tableName, self.messageid_field, modify_id)
             mm.execute(self.corpdb, self.dbCursor, alter, charset=self.encoding, use_unicode=self.use_unicode)
-        
-        mm.standardizeTable(self.corpdb, self.dbCursor, tableName, collate=dlac.DEF_COLLATIONS[self.encoding.lower()], engine=dlac.DEF_MYSQL_ENGINE, charset=self.encoding, use_unicode=self.use_unicode)
-        mm.disableTableKeys(self.corpdb, self.dbCursor, tableName, charset=self.encoding, use_unicode=self.use_unicode)
-    
-        columnNames = list(mm.getTableColumnNameTypes(self.corpdb, self.dbCursor, self.corptable, charset=self.encoding, use_unicode=self.use_unicode).keys())
+
+        mm.standardizeTable(self.corpdb, self.dbCursor, tableName, collate=dlac.DEF_COLLATIONS[self.encoding.lower()], engine=dlac.DEF_MYSQL_ENGINE, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
+        mm.disableTableKeys(self.corpdb, self.dbCursor, tableName, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
+
+        columnNames = list(mm.getTableColumnNameTypes(self.corpdb, self.dbCursor, self.corptable, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file).keys())
         messageIndex = columnNames.index(self.message_field)
         messageIdIndex = columnNames.index(self.messageid_field)
         return columnNames, messageIndex, messageIdIndex
 
-    def __findAllGroups(self):
+    def _findAllGroups(self):
         usql = """SELECT %s FROM %s GROUP BY %s""" % (self.correl_field, self.corptable, self.correl_field)
-        cfRows = [r[0] for r in mm.executeGetList(self.corpdb, self.dbCursor, usql, charset=self.encoding, use_unicode=self.use_unicode)]
+        cfRows = [r[0] for r in mm.executeGetList(self.corpdb, self.dbCursor, usql, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)]
         dlac.warn("finding messages for %d '%s's"%(len(cfRows), self.correl_field))
         return cfRows
 
-    def __getMsgsForGroups(self, groups, columnNames, messageIndex):
+    def _getMsgsForGroups(self, groups, columnNames, messageIndex):
         sql = """SELECT %s from %s where %s IN ('%s')""" % (','.join(columnNames), self.corptable, self.correl_field, "','".join(str(g) for g in groups))
-        rows = list(mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode))#, False)
+        rows = list(mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file))#, False)
         return rows
 
-    def __writeMsgsForGroups(self, rows, parses, messageIndex, tableName, columnNames):
+    def _writeMsgsForGroups(self, rows, parses, messageIndex, tableName, columnNames):
         insert_idx_start = 0
         insert_idx_end = dlac.MYSQL_BATCH_INSERT_SIZE
 
@@ -89,10 +95,42 @@ class MessageTransformer(DLAWorker):
 
         while insert_idx_start < len(rows):
             dataToWrite = rows[insert_idx_start:min(insert_idx_end, len(rows))]
-            mm.executeWriteMany(self.corpdb, self.dbCursor, sql, dataToWrite, writeCursor=self.dbConn.cursor(), charset=self.encoding, use_unicode=self.use_unicode)
+            mm.executeWriteMany(self.corpdb, self.dbCursor, sql, dataToWrite, writeCursor=self.dbConn.cursor(), charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
             insert_idx_start += dlac.MYSQL_BATCH_INSERT_SIZE
             insert_idx_end += dlac.MYSQL_BATCH_INSERT_SIZE
 
+    def _addTransformedMessages(self, tableName, transformation_func):
+        
+        #Create Table:
+        columnNames, messageIndex, messageIdIndex = self._createTable(tableName, modify='LONGTEXT')
+
+        #find all groups
+        cfRows = self._findAllGroups()
+
+        #iterate through groups in chunks
+        groupsWritten = 0
+        for groups in dlac.chunks(cfRows, self.groupsAtTime):
+
+            #get msgs for groups:
+            rows = self._getMsgsForGroups(groups, columnNames, messageIndex)
+            messages = [r[messageIndex] for r in rows]
+
+            if messages:
+                #tokenize msgs:
+                parses = [json.dumps(transformation_func(m)) for m in messages]
+                self._writeMsgsForGroups(rows, parses, messageIndex, tableName, columnNames)
+
+                groupsWritten += self.groupsAtTime
+                if groupsWritten % 100 == 0:
+                    dlac.warn("  %.1fk %ss' messages tagged and written" % (groupsWritten/float(1000), self.correl_field))
+            else:
+                dlac.warn("   Warning: No messages for:" + str(groups))
+
+        #re-enable keys:
+        mm.enableTableKeys(self.corpdb, self.dbCursor, tableName, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
+
+        return tableName
+
     def insertLDARows(self, ldas, tableName, columnNames, messageIndex, messageIdIndex):
         """?????
 
@@ -115,7 +153,7 @@ class MessageTransformer(DLAWorker):
             ','.join(columnNames), self.corptable, self.messageid_field,
             "','".join(message_ids))
 
-        rows = list(mm.executeGetList(self.corpdb, self.dbCursor, sql, False, charset=self.encoding, use_unicode=self.use_unicode))
+        rows = list(mm.executeGetList(self.corpdb, self.dbCursor, sql, False, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file))
 
         #generate row data:
         newRows = []
@@ -128,10 +166,10 @@ class MessageTransformer(DLAWorker):
         #insert
         sql = """INSERT INTO """+tableName+""" ("""+', '.join(columnNames)+\
             """) VALUES ("""  +", ".join(['%s']*len(columnNames)) + """)"""
-        mm.executeWriteMany(self.corpdb, self.dbCursor, sql, newRows, writeCursor=self.dbConn.cursor(), charset=self.encoding, use_unicode=self.use_unicode)
+        mm.executeWriteMany(self.corpdb, self.dbCursor, sql, newRows, writeCursor=self.dbConn.cursor(), charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
 
 
-    def addLDAMessages(self, ldaStatesFile):
+    def addLDAMessages(self, ldaStatesFile, ldaStatesName=None):
         """Creates a LDA topic version of message table
 
         Parameters
@@ -144,12 +182,14 @@ class MessageTransformer(DLAWorker):
         tableName : str
             Name of LDA message table: corptable_lda$ldaStatesFileBaseName
         """
+        print('LDA states file: {}'.format(ldaStatesFile))
         fin = open(ldaStatesFile, 'r') #done first so throws error if not existing
-        baseFileName = os.path.splitext(os.path.basename(ldaStatesFile))[0].replace('-', '_')
-        tableName = "%s_lda$%s" %(self.corptable, baseFileName)
+        if ldaStatesName is None:
+            ldaStatesName = os.path.splitext(os.path.basename(ldaStatesFile))[0].replace('-', '_')
+        tableName = "%s_lda$%s" %(self.corptable, ldaStatesName)
 
         #Create Table:
-        columnNames, messageIndex, messageIdIndex = self.createTable(tableName, modify='LONGTEXT')
+        columnNames, messageIndex, messageIdIndex = self._createTable(tableName, modify='LONGTEXT')
 
         commentLine = re.compile('^\#')
         ldaColumnLabels = ['doc', 'message_id', 'index', 'term_id', 'term', 'topic_id']
@@ -180,7 +220,7 @@ class MessageTransformer(DLAWorker):
         self.insertLDARows(ldas, tableName, columnNames, messageIndex, messageIdIndex)
 
         #re-enable keys:
-        mm.enableTableKeys(self.corpdb, self.dbCursor, tableName, charset=self.encoding, use_unicode=self.use_unicode)
+        mm.enableTableKeys(self.corpdb, self.dbCursor, tableName, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
 
         return tableName
 
@@ -194,38 +234,8 @@ class MessageTransformer(DLAWorker):
         """
         tableName = "%s_tok" %(self.corptable)
         tokenizer = Tokenizer(use_unicode=self.use_unicode)
-
-        #Create Table:
-        columnNames, messageIndex, messageIdIndex = self.__createTable(tableName, modify='LONGTEXT')
-
-        #find all groups
-        cfRows = self.__findAllGroups()
-
-        #iterate through groups in chunks
-
-        if self.correl_field == 'message_id':#more at a time when messages:
-            self.groupsAtTime = 2000
-        groupsWritten = 0
-        for groups in dlac.chunks(cfRows, self.groupsAtTime):
-
-            #get msgs for groups:
-            rows = self.__getMsgsForGroups(groups, columnNames, messageIndex)
-            messages = [r[messageIndex] for r in rows]
-
-            if messages:
-                #tokenize msgs:
-                parses = [json.dumps(tokenizer.tokenize(m)) for m in messages]
-                self.__writeMsgsForGroups(rows, parses, messageIndex, tableName, columnNames)
-
-                groupsWritten += self.groupsAtTime
-                if groupsWritten % 100 == 0:
-                    dlac.warn("  %.1fk %ss' messages tagged and written" % (groupsWritten/float(1000), self.correl_field))
-            else:
-                dlac.warn("   Warning: No messages for:" + str(groups))
-
-        #re-enable keys:
-        mm.enableTableKeys(self.corpdb, self.dbCursor, tableName, charset=self.encoding, use_unicode=self.use_unicode)
-
+        self._addTransformedMessages(tableName, tokenizer.tokenize)
+        
         return tableName
 
     def addSegmentedMessages(self, model="ctb", tmpdir="/tmp"):
@@ -241,7 +251,7 @@ class MessageTransformer(DLAWorker):
 
         assert model.lower() in ["ctb", "pku"], "Available models for segmentation are CTB or PKU"
         sql = "select %s, %s from %s" % (self.messageid_field, self.message_field, self.corptable)
-        rows = mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode)
+        rows = mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
 
         tmpfile = tmpdir+"/tmpChineseUnsegmented.txt"
         tmpfile_seg = tmpdir+"/tmpChineseSegmented.txt"
@@ -278,7 +288,7 @@ class MessageTransformer(DLAWorker):
         sql = "SELECT column_name, column_type FROM INFORMATION_SCHEMA.COLUMNS "
         sql += "WHERE table_name = '%s' AND COLUMN_NAME in ('%s', '%s') and table_schema = '%s'" % (
             self.corptable, self.message_field, self.messageid_field, self.corpdb)
-        types = {k:v for k,v in mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode)}
+        types = {k:v for k,v in mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)}
         sql2 = "CREATE TABLE %s (" % (self.corptable+"_seg")
         sql2 += "%s %s primary key, %s %s " % (self.messageid_field,
                                                  types[self.messageid_field],
@@ -286,11 +296,11 @@ class MessageTransformer(DLAWorker):
                                                  types[self.message_field],
                                                  )
         sql2 += ")"
-        mm.execute(self.corpdb, self.dbCursor, "drop table if exists "+tableName, charset=self.encoding, use_unicode=self.use_unicode)
-        mm.execute(self.corpdb, self.dbCursor, sql2, charset=self.encoding, use_unicode=self.use_unicode)
-        mm.standardizeTable(self.corpdb, self.dbCursor, tableName, collate=dlac.DEF_COLLATIONS[self.encoding.lower()], engine=dlac.DEF_MYSQL_ENGINE, charset=self.encoding, use_unicode=self.use_unicode)
+        mm.execute(self.corpdb, self.dbCursor, "drop table if exists "+tableName, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
+        mm.execute(self.corpdb, self.dbCursor, sql2, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
+        mm.standardizeTable(self.corpdb, self.dbCursor, tableName, collate=dlac.DEF_COLLATIONS[self.encoding.lower()], engine=dlac.DEF_MYSQL_ENGINE, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
         alter = """ALTER TABLE %s MODIFY %s LONGTEXT""" % (tableName, self.message_field)
-        mm.execute(self.corpdb, self.dbCursor, alter, charset=self.encoding, use_unicode=self.use_unicode)
+        mm.execute(self.corpdb, self.dbCursor, alter, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
 
         sql = "INSERT INTO %s " % (tableName)
         sql += " VALUES (%s, %s)"
@@ -298,7 +308,7 @@ class MessageTransformer(DLAWorker):
         totalLength = len(new_rows)
         for l in range(0, totalLength, N):
             print("Inserting rows (%5.2f%% done)" % (float(min(l+N,totalLength))*100/totalLength))
-            mm.executeWriteMany(self.corpdb, self.dbCursor, sql, new_rows[l:l+N], writeCursor=self.dbConn.cursor(), charset=self.encoding)
+            mm.executeWriteMany(self.corpdb, self.dbCursor, sql, new_rows[l:l+N], writeCursor=self.dbConn.cursor(), charset=self.encoding, mysql_config_file=self.mysql_config_file)
 
     def addTweetPOSMessages(self):
         """Creates a POS tagged (by TweetNLP) version of the message table
@@ -314,35 +324,9 @@ class MessageTransformer(DLAWorker):
         except NameError:
             dlac.warn("Method not available without TweetNLP interface")
             raise
+        self._addTransformedMessages(tableName, tagger.tag)
 
-        #Create Table:
-        columnNames, messageIndex, messageIdIndex = self.__createTable(tableName)
-
-        #find all groups
-        cfRows = self.__findAllGroups()
-
-        #iterate through groups in chunks
-        groupsWritten = 0
-        for groups in dlac.chunks(cfRows, self.groupsAtTime):
-
-            #get msgs for groups:
-            rows = self.__getMsgsForGroups(groups, columnNames, messageIndex)
-            messages = [r[messageIndex] for r in rows]
-
-            if messages:
-                #tokenize msgs:
-                parses = [json.dumps(tagger.tag(m)) for m in messages]
-                self.__writeMsgsForGroups(rows, parses, messageIndex, tableName, columnNames)
-
-                groupsWritten += self.groupsAtTime
-                if groupsWritten % 100 == 0:
-                    dlac.warn("  %.1fk %ss' messages tagged and written" % (groupsWritten/float(1000), self.correl_field))
-            else:
-                dlac.warn("   Warning: No messages for:" + str(groups))
-
-        #re-enable keys:
-        mm.enableTableKeys(self.corpdb, self.dbCursor, tableName, charset=self.encoding, use_unicode=self.use_unicode)
-
+        
         return tableName
 
     def addTweetTokenizedMessages(self):
@@ -359,38 +343,11 @@ class MessageTransformer(DLAWorker):
         except NameError:
             dlac.warn("Method not available without TweetNLP interface")
             raise
-
-        #Create Table:
-        columnNames, messageIndex, messageIdIndex = self.__createTable(tableName)
-
-        #find all groups
-        cfRows = self.__findAllGroups()
-
-        #iterate through groups in chunks
-        groupsWritten = 0
-        for groups in dlac.chunks(cfRows, self.groupsAtTime):
-
-            #get msgs for groups:
-            rows = self.__getMsgsForGroups(groups, columnNames, messageIndex)
-            messages = [r[messageIndex] for r in rows]
-
-            if messages:
-                #tokenize msgs:
-                parses = [json.dumps(tokenizer.tokenize(m)) for m in messages]
-                self.__writeMsgsForGroups(rows, parses, messageIndex, tableName, columnNames)
-
-                groupsWritten += self.groupsAtTime
-                if groupsWritten % 100 == 0:
-                    dlac.warn("  %.1fk %ss' messages tagged and written" % (groupsWritten/float(1000), self.correl_field))
-            else:
-                dlac.warn("   Warning: No messages for:" + str(groups))
-
-        #re-enable keys:
-        mm.enableTableKeys(self.corpdb, self.dbCursor, tableName, charset=self.encoding, use_unicode=self.use_unicode)
+        self._addTransformedMessages(tableName, tokenizer.tokenize)
 
         return tableName
 
-    def addSentTokenizedMessages(self, sentPerRow = False, cleanMessages = None):
+    def addSentTokenizedMessages(self, sentPerRow = False, cleanMessages = None, newlinesToPeriods=True):
         """Creates a sentence tokenized version of message table
 
         Returns
@@ -406,11 +363,13 @@ class MessageTransformer(DLAWorker):
 
         #Create Table:
         if sentPerRow:
-            modify = 'VARCHAR(64)'
+            modify = ''#don't modify message_column
+            modify_id = 'VARCHAR(128)'#modify id column
         else:
             modify = ''
-        columnNames, messageIndex, messageIdIndex = self.__createTable(tableName, modify)
-        
+            modify_id = ''
+        columnNames, messageIndex, messageIdIndex = self._createTable(tableName, modify, modify_id)
+                                                                    
         if cleanMessages:
 
             ### get lexical normalization dictionaries
@@ -418,16 +377,16 @@ class MessageTransformer(DLAWorker):
             
             # Han, Bo  and  Cook, Paul  and  Baldwin, Timothy, 2012
             sql = """select word, norm from %s.%s""" % (dlac.DEF_LEXICON_DB, "han_bo_emnlp_dict")
-            normalizeDict.update(dict(list(mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode))))
+            normalizeDict.update(dict(list(mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file))))
             
             # Fei Liu, Fuliang Weng, Bingqing Wang, Yang Liu, 2011 
             # Fei Liu, Fuliang Weng, Xiao Jiang, 2012
             sql = """select word, norm from %s.%s""" % (dlac.DEF_LEXICON_DB, "liu_weng_test_set_3802")
-            normalizeDict.update(dict(list(mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode))))
+            normalizeDict.update(dict(list(mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file))))
         
 
         #find all groups
-        cfRows = self.__findAllGroups()
+        cfRows = self._findAllGroups()
 
         #iterate through groups in chunks
         groupsWritten = 0
@@ -435,9 +394,7 @@ class MessageTransformer(DLAWorker):
             sentRows = list()
             
             #get msgs for groups:
-            sql = """SELECT %s from %s where %s IN ('%s')""" % (','.join(columnNames), self.corptable, self.correl_field, "','".join(str(g) for g in groups))
-            rows = list(mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode))#, False)
-            rows = [r for r in rows if r[messageIndex]]
+            rows = self._getMsgsForGroups(groups, columnNames, messageIndex)
             messages = [r[messageIndex] for r in rows]
 
             if messages:
@@ -445,17 +402,26 @@ class MessageTransformer(DLAWorker):
                 insert_idx_end = dlac.MYSQL_BATCH_INSERT_SIZE
                 #tokenize msgs:
                 # parses = map(lambda m: json.dumps(sentDetector.tokenize(tc.removeNonAscii(tc.treatNewlines(m.strip())))), messages)
-                parses = None
-                if self.use_unicode:
+                parses = []
+                for m in messages:
+                    parse = m.strip()
                     if cleanMessages:
-                        parses = [json.dumps(sentDetector.tokenize((tc.sentenceNormalization(m.strip(), normalizeDict, self.use_unicode)))) for m in messages]
-                    else:
-                        parses = [json.dumps(sentDetector.tokenize(tc.removeNonUTF8(tc.treatNewlines(m.strip())))) for m in messages]
-                        #parses = [json.dumps(sentDetector.tokenize(tc.removeNonUTF8(m.strip()))) for m in messages]
-                else:
-                    parses = [json.dumps(sentDetector.tokenize(tc.removeNonUTF8(tc.treatNewlines(m.strip())))) for m in messages]
-                    #parses = [json.dumps(sentDetector.tokenize(tc.removeNonUTF8(m.strip()))) for m in messages]
-                    #parses = [json.dumps(sentDetector.tokenize(tc.removeNonAscii(tc.treatNewlines(m.strip())))) for m in messages]
+                        parse = tc.sentenceNormalization(parse, normalizeDict, self.use_unicode)
+                    parse = tc.removeNonUTF8(tc.treatNewlines(parse))
+                    if newlinesToPeriods:
+                        parse = parse.replace('<NEWLINE>', '.')
+                    parse = sentDetector.tokenize((parse))
+                    parses.append(json.dumps(parse))
+                # if self.use_unicode:
+                #     if cleanMessages:
+                #         parses = [json.dumps(sentDetector.tokenize((tc.sentenceNormalization(m.strip(), normalizeDict, self.use_unicode)))) for m in messages]
+                #     else:
+                #         parses = [json.dumps(sentDetector.tokenize(tc.removeNonUTF8(tc.treatNewlines(m.strip())))) for m in messages]
+                #         #parses = [json.dumps(sentDetector.tokenize(tc.removeNonUTF8(m.strip()))) for m in messages]
+                # else:
+                #     parses = [json.dumps(sentDetector.tokenize(tc.removeNonUTF8(tc.treatNewlines(m.strip())))) for m in messages]
+                #     #parses = [json.dumps(sentDetector.tokenize(tc.removeNonUTF8(m.strip()))) for m in messages]
+                #     #parses = [json.dumps(sentDetector.tokenize(tc.removeNonAscii(tc.treatNewlines(m.strip())))) for m in messages]
 
                 #add msgs into new tables
                 sql = """INSERT INTO """+tableName+""" ("""+', '.join(columnNames)+\
@@ -465,7 +431,7 @@ class MessageTransformer(DLAWorker):
                     if sentPerRow:
                         for j, parse in enumerate(ast.literal_eval(parses[i]), 1):
                             sentRows.append(list(rows[i]))
-                            sentRows[-1][messageIDIndex] = str(rows[i][messageIDIndex]) + "_" + str(j).zfill(2)
+                            sentRows[-1][messageIdIndex] = str(rows[i][messageIdIndex]) + "_" + str(j).zfill(2)
                             sentRows[-1][messageIndex] = parse
                     elif i < len(parses):
                         sentRows.append(rows[i])#debug: take out copy if eveyrthing ok to run faster
@@ -482,10 +448,10 @@ class MessageTransformer(DLAWorker):
                         #         print("!NO PARSE!")
                         # sys.exit(1)
 
-                while insert_idx_start < len(rows):
-                    dataToWrite = sentRows[insert_idx_start:min(insert_idx_end, len(rows))]
-                    #_warn("Inserting rows %d to %d... " % (insert_idx_start, insert_idx_end))
-                    mm.executeWriteMany(self.corpdb, self.dbCursor, sql, dataToWrite, writeCursor=self.dbConn.cursor(), charset=self.encoding, use_unicode=self.use_unicode)
+                while insert_idx_start < len(sentRows):
+                    dataToWrite = sentRows[insert_idx_start:min(insert_idx_end, len(sentRows))]
+                    #dlac.warn("Inserting rows %d to %d... " % (insert_idx_start, insert_idx_end))
+                    mm.executeWriteMany(self.corpdb, self.dbCursor, sql, dataToWrite, writeCursor=self.dbConn.cursor(), charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
                     insert_idx_start += dlac.MYSQL_BATCH_INSERT_SIZE
                     insert_idx_end += dlac.MYSQL_BATCH_INSERT_SIZE
                 
@@ -496,7 +462,7 @@ class MessageTransformer(DLAWorker):
                 dlac.warn("   Warning: No messages for:" + str(groups))
 
         #re-enable keys:
-        mm.enableTableKeys(self.corpdb, self.dbCursor, tableName, charset=self.encoding, use_unicode=self.use_unicode)
+        mm.enableTableKeys(self.corpdb, self.dbCursor, tableName, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
 
         return tableName
 
@@ -513,7 +479,7 @@ class MessageTransformer(DLAWorker):
         imp.reload(sys)
         if not self.use_unicode: sys.setdefaultencoding('utf8')
         sql = """SELECT %s, %s  from %s""" % (self.messageid_field, self.message_field,self.corptable+'_tok')
-        messagesEnc = mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode)
+        messagesEnc = mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
         try:
             messagesTok = [(m[0], json.loads(m[1])) for m in messagesEnc]
         except ValueError:
@@ -522,7 +488,7 @@ class MessageTransformer(DLAWorker):
         whiteSet = None
         if whiteListFeatTable:
             sql = "SELECT distinct feat FROM %s " % whiteListFeatTable[0]
-            whiteSet = set([s[0] for s in mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode)])
+            whiteSet = set([s[0] for s in mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)])
 
         f = open(filename, 'w')
         for m in messagesTok:
@@ -564,7 +530,7 @@ class MessageTransformer(DLAWorker):
                 rows[i] = list(rows[i])
                 rows[i][messageIndex] = str(parses[i][pt])
 
-            mm.executeWriteMany(self.corpdb, self.dbCursor, sql, rows, writeCursor=self.dbConn.cursor(), charset=self.encoding)
+            mm.executeWriteMany(self.corpdb, self.dbCursor, sql, rows, writeCursor=self.dbConn.cursor(), charset=self.encoding, mysql_config_file=self.mysql_config_file)
         return True
     
     def addParsedMessages(self):
@@ -581,17 +547,17 @@ class MessageTransformer(DLAWorker):
         #Create Tables: (TODO make continue)
         for t, name in list(tableNames.items()):
             sql = "CREATE TABLE IF NOT EXISTS %s like %s" % (name, self.corptable)
-            mm.execute(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode)
-            mm.standardizeTable(self.corpdb, self.dbCursor, name, collate=dlac.DEF_COLLATIONS[self.encoding.lower()], engine=dlac.DEF_MYSQL_ENGINE, charset=self.encoding, use_unicode=self.use_unicode)
-            mm.enableTableKeys(self.corpdb, self.dbCursor, name, charset=self.encoding, use_unicode=self.use_unicode)#just incase interrupted, so we can find un-parsed groups
+            mm.execute(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
+            mm.standardizeTable(self.corpdb, self.dbCursor, name, collate=dlac.DEF_COLLATIONS[self.encoding.lower()], engine=dlac.DEF_MYSQL_ENGINE, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
+            mm.enableTableKeys(self.corpdb, self.dbCursor, name, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)#just incase interrupted, so we can find un-parsed groups
 
         #Find column names:
-        columnNames = list(mm.getTableColumnNameTypes(self.corpdb, self.dbCursor, self.corptable, charset=self.encoding, use_unicode=self.use_unicode).keys())
+        columnNames = list(mm.getTableColumnNameTypes(self.corpdb, self.dbCursor, self.corptable, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file).keys())
         messageIndex = columnNames.index(self.message_field)
 
         #find if parsed table already has rows:
         countsql = """SELECT count(*) FROM %s""" % (tableNames[parseTypes[0]])
-        cnt = mm.executeGetList(self.corpdb, self.dbCursor, countsql, charset=self.encoding, use_unicode=self.use_unicode)[0][0]
+        cnt = mm.executeGetList(self.corpdb, self.dbCursor, countsql, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)[0][0]
 
         #find all groups that are not already inserted
         usql = """SELECT %s FROM %s GROUP BY %s""" % (self.correl_field, self.corptable, self.correl_field)
@@ -599,12 +565,12 @@ class MessageTransformer(DLAWorker):
             usql = """SELECT a.%s FROM %s AS a LEFT JOIN %s AS b ON a.%s = b.%s WHERE b.%s IS NULL group by a.%s""" % (
                 self.correl_field, self.corptable, tableNames[parseTypes[0]], self.messageid_field, self.messageid_field, self.messageid_field, self.correl_field)
         #msgs = 0#keeps track of the number of messages read
-        cfRows = [r[0] for r in mm.executeGetList(self.corpdb, self.dbCursor, usql, charset=self.encoding, use_unicode=self.use_unicode)]
+        cfRows = [r[0] for r in mm.executeGetList(self.corpdb, self.dbCursor, usql, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)]
         dlac.warn("parsing messages for %d '%s's"%(len(cfRows), self.correl_field))
 
         #disable keys (waited until after finding groups)
         for t, name in list(tableNames.items()):
-            mm.disableTableKeys(self.corpdb, self.dbCursor, name, charset=self.encoding, use_unicode=self.use_unicode)
+            mm.disableTableKeys(self.corpdb, self.dbCursor, name, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
 
         #iterate through groups in chunks
         if not any(field in self.correl_field.lower() for field in ["mess", "msg"]) or self.correl_field.lower().startswith("id"):
@@ -622,8 +588,7 @@ class MessageTransformer(DLAWorker):
         activePs = set()
         for groups in dlac.chunks(cfRows, self.groupsAtTime):
             #get msgs for groups:
-            sql = """SELECT %s from %s where %s IN ('%s')""" % (','.join(columnNames), self.corptable, self.correl_field, "','".join(str(g) for g in groups))
-            rows = list(mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode))
+            rows = self._getMsgsForGroups(groups, columnNames, messageIndex)
             rows = [row for row in rows if row[messageIndex] and not row[messageIndex].isspace()]
             messages = [r[messageIndex] for r in rows]
             messages = [m if m else '_' for m in messages]
@@ -662,6 +627,6 @@ class MessageTransformer(DLAWorker):
 
         #re-enable keys:
         for t, name in list(tableNames.items()):
-            mm.enableTableKeys(self.corpdb, self.dbCursor, name, charset=self.encoding, use_unicode=self.use_unicode)
+            mm.enableTableKeys(self.corpdb, self.dbCursor, name, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)
 
         return tableNames
