@@ -1,6 +1,9 @@
 import sys
 import re
-import MySQLdb
+try:
+    import MySQLdb
+except:
+    pass
 import pandas as pd
 import numpy as np
 from math import isclose
@@ -11,6 +14,9 @@ from . import dlaConstants as dlac
 from .mysqlmethods import mysqlMethods as mm
 from .mysqlmethods.mysql_iter_funcs import get_db_engine
 
+from .database.query import QueryBuilder
+from .database.dataEngine import DataEngine
+
 class OutcomeGetter(DLAWorker):
     """Deals with outcome tables
 
@@ -76,10 +82,11 @@ class OutcomeGetter(DLAWorker):
         """
         parser = SafeConfigParser()
         parser.read(initFile)
+        db_type = parser.get('constants','db_type') if parser.has_option('constants','db_type') else dlac.DB_TYPE
         corpdb = parser.get('constants','corpdb') if parser.has_option('constants','corpdb') else dlac.DEF_CORPDB
         corptable = parser.get('constants','corptable') if parser.has_option('constants','corptable') else dlac.DEF_CORPTABLE
         correl_field = parser.get('constants','correl_field') if parser.has_option('constants','correl_field') else dlac.DEF_CORREL_FIELD
-        mysql_host = parser.get('constants','mysql_host') if parser.has_option('constants','mysql_host') else dlac.MYSQL_HOST
+        mysql_config_file = parser.get('constants','mysql_config_file') if parser.has_option('constants','mysql_config_file') else dlac.MYSQL_CONFIG_FILE
         message_field = parser.get('constants','message_field') if parser.has_option('constants','message_field') else dlac.DEF_MESSAGE_FIELD
         messageid_field = parser.get('constants','messageid_field') if parser.has_option('constants','messageid_field') else dlac.DEF_MESSAGEID_FIELD
         encoding = parser.get('constants','encoding') if parser.has_option('constants','encoding') else dlac.DEF_ENCODING
@@ -99,11 +106,11 @@ class OutcomeGetter(DLAWorker):
         featureMappingTable = parser.get('constants','featlabelmaptable') if parser.has_option('constants','featlabelmaptable') else ''
         featureMappingLex = parser.get('constants','featlabelmaplex') if parser.has_option('constants','featlabelmaplex') else ''
         wordTable = parser.get('constants','wordTable') if parser.has_option('constants','wordTable') else None
-        return cls(corpdb=corpdb, corptable=corptable, correl_field=correl_field, mysql_host=mysql_host, message_field=message_field, messageid_field=messageid_field, encoding=encoding, use_unicode=use_unicode, lexicondb=lexicondb, outcome_table=outcome_table, outcome_value_fields=outcome_value_fields, outcome_controls=outcome_controls, outcome_interaction=outcome_interaction, outcome_categories=outcome_categories, multiclass_outcome=multiclass_outcome, group_freq_thresh=group_freq_thresh, low_variance_thresh=low_variance_thresh, featureMappingTable=featureMappingTable, featureMappingLex=featureMappingLex, wordTable=wordTable)
+        return cls(db_type=db_type, corpdb=corpdb, corptable=corptable, correl_field=correl_field, mysql_config_file=mysql_config_file, message_field=message_field, messageid_field=messageid_field, encoding=encoding, use_unicode=use_unicode, lexicondb=lexicondb, outcome_table=outcome_table, outcome_value_fields=outcome_value_fields, outcome_controls=outcome_controls, outcome_interaction=outcome_interaction, outcome_categories=outcome_categories, multiclass_outcome=multiclass_outcome, group_freq_thresh=group_freq_thresh, low_variance_thresh=low_variance_thresh, featureMappingTable=featureMappingTable, featureMappingLex=featureMappingLex, wordTable=wordTable)
     
 
-    def __init__(self, corpdb=dlac.DEF_CORPDB, corptable=dlac.DEF_CORPTABLE, correl_field=dlac.DEF_CORREL_FIELD, mysql_host=dlac.MYSQL_HOST, message_field=dlac.DEF_MESSAGE_FIELD, messageid_field=dlac.DEF_MESSAGEID_FIELD, encoding=dlac.DEF_ENCODING, use_unicode=dlac.DEF_UNICODE_SWITCH, lexicondb = dlac.DEF_LEXICON_DB, outcome_table=dlac.DEF_OUTCOME_TABLE, outcome_value_fields=[dlac.DEF_OUTCOME_FIELD], outcome_controls = dlac.DEF_OUTCOME_CONTROLS, outcome_interaction = dlac.DEF_OUTCOME_CONTROLS, outcome_categories = [], multiclass_outcome = [], group_freq_thresh = None, low_variance_thresh = dlac.DEF_LOW_VARIANCE_THRESHOLD, featureMappingTable='', featureMappingLex='', wordTable = None, fold_column = None):
-        super(OutcomeGetter, self).__init__(corpdb, corptable, correl_field, mysql_host, message_field, messageid_field, encoding, use_unicode, lexicondb, wordTable = wordTable)
+    def __init__(self, db_type=dlac.DB_TYPE, corpdb=dlac.DEF_CORPDB, corptable=dlac.DEF_CORPTABLE, correl_field=dlac.DEF_CORREL_FIELD, mysql_config_file=dlac.MYSQL_CONFIG_FILE, message_field=dlac.DEF_MESSAGE_FIELD, messageid_field=dlac.DEF_MESSAGEID_FIELD, encoding=dlac.DEF_ENCODING, use_unicode=dlac.DEF_UNICODE_SWITCH, lexicondb = dlac.DEF_LEXICON_DB, outcome_table=dlac.DEF_OUTCOME_TABLE, outcome_value_fields=[dlac.DEF_OUTCOME_FIELD], outcome_controls = dlac.DEF_OUTCOME_CONTROLS, outcome_interaction = dlac.DEF_OUTCOME_CONTROLS, outcome_categories = [], multiclass_outcome = [], group_freq_thresh = None, low_variance_thresh = dlac.DEF_LOW_VARIANCE_THRESHOLD, featureMappingTable='', featureMappingLex='', wordTable = None, fold_column = None):
+        super(OutcomeGetter, self).__init__(db_type, corpdb, corptable, correl_field, mysql_config_file, message_field, messageid_field, encoding, use_unicode, lexicondb, wordTable = wordTable)
         self.outcome_table = outcome_table
 
         if isinstance(outcome_value_fields, str):
@@ -111,7 +118,7 @@ class OutcomeGetter(DLAWorker):
 
         if outcome_value_fields and len(outcome_value_fields) > 0 and outcome_value_fields[0] == '*':#handle wildcard fields
             newOutcomeFields = []
-            for name, typ in mm.getTableColumnNameTypes(self.corpdb, self.dbCursor, outcome_table).items():
+            for name, typ in mm.getTableColumnNameTypes(self.corpdb, self.dbCursor, outcome_table, mysql_config_file=self.mysql_config_file).items():
                 typ = re.sub(r'\([0-9\,]*\)\s*$', '', typ)
                 if typ.split()[0].upper() in self._mysqlNumeric:
                     newOutcomeFields.append(name)
@@ -119,7 +126,7 @@ class OutcomeGetter(DLAWorker):
 
         if outcome_controls and len(outcome_controls) > 0 and outcome_controls[0] == '*':#handle wildcard fields
             newOutcomeFields = []
-            for name, typ in mm.getTableColumnNameTypes(self.corpdb, self.dbCursor, outcome_table).items():
+            for name, typ in mm.getTableColumnNameTypes(self.corpdb, self.dbCursor, outcome_table, mysql_config_file=self.mysql_config_file).items():
                 typ = re.sub(r'\([0-9\,]*\)\s*$', '', typ)
                 if typ.split()[0].upper() in self._mysqlNumeric:
                     newOutcomeFields.append(name)
@@ -147,7 +154,7 @@ class OutcomeGetter(DLAWorker):
 
     def copy(self):
         self.__dict__
-        newObj = OutcomeGetter(self.corpdb, self.corptable, self.correl_field, self.mysql_host, self.message_field, self.messageid_field)
+        newObj = OutcomeGetter(self.db_type, self.corpdb, self.corptable, self.correl_field, self.mysql_config_file, self.message_field, self.messageid_field)
         for k, v in self.__dict__.items():
             newObj.__dict__[k] = v
         return newObj
@@ -167,7 +174,7 @@ class OutcomeGetter(DLAWorker):
         return feat_to_label
 
     def createOutcomeTable(self, tablename, dataframe, ifExists='fail'):
-        eng = get_db_engine(self.corpdb, self.mysql_host)
+        eng = get_db_engine(self.corpdb, mysql_config_file=self.mysql_config_file)
         dtype ={}
         if isinstance(dataframe.index[0], str):
             import sqlalchemy
@@ -188,7 +195,7 @@ class OutcomeGetter(DLAWorker):
             if not includeNull:
                 wheres.append("%s IS NOT NULL" % outcome)
             sql += ' WHERE ' + ' AND '.join(wheres)
-        return [v[0] for v in mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding)]
+        return [v[0] for v in mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, mysql_config_file=self.mysql_config_file)]
 
     def getDistinctOutcomeValueCounts(self, outcome = None, requireControls = False, includeNull = True, where = ''):
         """returns a dict of (outcome_value, count)"""
@@ -206,7 +213,7 @@ class OutcomeGetter(DLAWorker):
             sql += ' WHERE ' + ' AND '.join(wheres)
             
         sql += ' group by %s ' % outcome
-        return dict(mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode))
+        return dict(mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file))
 
     def getDistinctOutcomeAndControlValueCounts(self, outcome = None, control = None, includeNull = True, where = ''):
         """returns a dict of (outcome_value, count)"""
@@ -227,7 +234,7 @@ class OutcomeGetter(DLAWorker):
             
         sql += ' group by %s, %s ' % (outcome, control)
         countDict = dict()
-        for (outcome, control, count) in mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode):
+        for (outcome, control, count) in mm.executeGetList(self.corpdb, self.dbCursor, sql, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file):
             if not outcome in countDict:
                 countDict[outcome] = dict()
             countDict[outcome][control] = count
@@ -237,9 +244,9 @@ class OutcomeGetter(DLAWorker):
     def getGroupAndOutcomeValues(self, outcomeField = None, where=''):
         """returns a list of (group_id, outcome_value) tuples"""
         if not outcomeField: outcomeField = self.outcome_value_fields[0]
-        sql = "select %s, %s from `%s` WHERE %s IS NOT NULL"%(self.correl_field, outcomeField, self.outcome_table, outcomeField)
-        if (where): sql += ' AND ' + where
-        return mm.executeGetList(self.corpdb, self.dbCursor, sql, False, charset=self.encoding, use_unicode=self.use_unicode)
+        query = self.qb.create_select_query(self.outcome_table).set_fields([self.correl_field, outcomeField]).where("%s IS NOT NULL"%outcomeField)
+        if (where): query.where(" AND "+ where)
+        return query.execute_query()
 
     def makeContingencyTable(self, featureGetter, featureValueField, outcome_filter_where='', feature_value_group_sum_min=0):
         """makes a contingency table from this outcome value, a featureGetter, and the desired column of the featureGetter, assumes both correl_field's are the same"""
@@ -292,7 +299,7 @@ class OutcomeGetter(DLAWorker):
 
         # finish the original query with the updated feature table
         sql += "FROM %s AS updatedFeatures GROUP BY %s"%(sql_filtered_feature_table_2, fg.correl_field)
-        return [distinctFeatureList, mm.executeGetList(self.corpdb, self.dbCursor, sql, False, charset=self.encoding, use_unicode=self.use_unicode)]
+        return [distinctFeatureList, mm.executeGetList(self.corpdb, self.dbCursor, sql, False, charset=self.encoding, use_unicode=self.use_unicode, mysql_config_file=self.mysql_config_file)]
 
     def makeBinnedOutcomeTable(self, buckets, mid_aom_list):
         """buckets is a list of tuples"""
@@ -328,7 +335,7 @@ enabled, so the total word count for your groups might be off
                             to_remove.append(outcomeField)
                             continue
                     except TypeError:
-                        dlac.warn("TypeError during variance check for %s, skipping step." % (outcomeField))
+                        dlac.warn("WARNING: Could not determine variance of %s. Skipping variance check." % (outcomeField))
                         outcomeVariance = 1
                     if isclose(outcomeVariance, 0.0) or outcomeVariance < self.low_variance_thresh:
                         del outcomes[outcomeField]
@@ -425,7 +432,7 @@ enabled, so the total word count for your groups might be off
                 groups = groups & set(outcomes[k].keys()) #always intersect with controls
             if self.outcome_categories:
                 for cat in cat_label_list:
-                    if all(outcomes[cat][group] == 0 for group in groups):
+                    if all(outcomes[cat][group] == 0 for group in groups) or all(outcomes[cat][group] == 1 for group in groups):
                         del outcomes[cat]
                         dlac.warn("Removing %s, no non-zero instances" % cat)
                         if cat in self.outcome_value_fields:
@@ -480,7 +487,7 @@ enabled, so the total word count for your groups might be off
     def getGroupAndOutcomeValuesAsDF(self, outcomeField = None, where=''):
         """returns a dataframe of (group_id, outcome_value)"""
         if not outcomeField: outcomeField = self.outcome_value_fields[0]
-        db_eng = get_db_engine(self.corpdb)
+        db_eng = get_db_engine(self.corpdb, mysql_config_file=self.mysql_config_file)
         sql = "select %s, %s from `%s` WHERE %s IS NOT NULL" % (self.correl_field, outcomeField, self.outcome_table, outcomeField)
         if (where): sql += ' WHERE ' + where
         index = self.correl_field
